package model;

import java.util.*;

/**
 * A complete pc that consists of a main board, one or more CPUs, one or more
 * hard drives, and one or more RAM blocks.
 * 
 * @author Kristian Torp, torp (at) cs (dot) aau (dot) dk
 * @version 1.0
 */

public class PC extends Component{
    /** The main board */
    private MainBoard mainBoard;
    /** The collection of cpus */
    private ArrayList cpus;
    /** The collection of disks */
    private ArrayList disks;
    /** The collection of RAM blocks */
    private ArrayList ramBlocks;

    /**
     * Default constructor
     */
    public PC() throws ModelException{
        super();
        mainBoard = null;
        cpus = new ArrayList();
        disks = new ArrayList();
        ramBlocks = new ArrayList();
    }
    
    /**
     * Default constructor
     */
    public PC(String make, String model) throws ModelException{
        super();
        setMake(make);
        setModel(model);
        mainBoard = null;
        cpus = new ArrayList();
        disks = new ArrayList();
        ramBlocks = new ArrayList();
    }
    

    /**
     * Constructor for "plain" PC with one cpu, one disk, and one RAM block.
     */
    public PC(String make, String model, MainBoard mainboard, CPU cpu, HardDrive disk, RAM ramBlock) 
        throws ModelException{
        this(make, model);
        addMainBoard(mainboard);
        addCPU(cpu);
        addHardDrive(disk);
        addRamBlock(ramBlock);
    }
    
    /**
     * String representation of object.
     * @param o the object to compare to
     * @see java.lang.Object#toString(java.lang.Object)
     */
    public String toString(){
        String 
          res  = "Make:        " + getMake()  + "\n";
          res += "Model:       " + getModel() + "\n";
          res += "Price:       " + getPrice() + "\n";

        // Add text main board
        res += "Main board" + "\n";
        res += mainBoard + "\n";
        
        // Add text for cpus
        Iterator cpuIter = cpus.iterator();
        int cpuInt = 0;
        while (cpuIter.hasNext()){
            CPU c = (CPU)cpuIter.next();
            res += "CPU " + cpuInt++ + "\n";
            res += c + "\n";
        }

        // Add text hard drives
        Iterator diskIter = disks.iterator();
        int diskInt = 0;
        while (diskIter.hasNext()){
            HardDrive d = (HardDrive)diskIter.next();
            res += "Hard drive " + diskInt++ + "\n";
            res += d + "\n";
        }

        // Add text RAM blocks
        Iterator ramIter = ramBlocks.iterator();
        int ramInt = 0;
        while (ramIter.hasNext()){
            RAM r = (RAM)ramIter.next();
            res += ramInt++ + "\n";
            res += r + "\n";
        }

        return res;
    }
    
    /**
     * Checks for equality
     * @param o the object to compare to
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if (!(o instanceof PC)){
            return false;
        }
        
        if (!super.equals(o)){
            return false;
        }
        
        // main boards equal
        MainBoard otherMainBoard = ((PC)o).getMainBoard();
        if (!mainBoard.equals(otherMainBoard)){
            return false;
        }
        
        // cpu equals
        int noCpuOther = ((PC)o).noCpus();
        if (noCpus() != noCpuOther){
            return false;
        }
        else{
            for(int i = 0; i < noCpus(); i++){
                try{
                    CPU hereCPU = getCPU(i);
                    CPU otherCPU = ((PC)o).getCPU(i);
                    if (!hereCPU.equals(otherCPU)){
                        return false;
                    }
                }
                catch(ModelException e){
                    throw new RuntimeException(e.getMessage());
                }
            }
        }

        // hard disks equal
        int noDisksOther = ((PC)o).noHardDrives();
        if (noHardDrives() != noDisksOther){
            return false;
        }
        else{
            for(int i = 0; i < noHardDrives(); i++){
                try{
                    HardDrive hereDisk = getHardDrive(i);
                    HardDrive otherDisk = ((PC)o).getHardDrive(i);
                    if (!hereDisk.equals(otherDisk)){
                        return false;
                    }
                }
                catch(ModelException e){
                    throw new RuntimeException(e.getMessage());
                }
            }
        }
        
        // ram blocks equal
        int noRamBlocksOther = ((PC)o).noRamBlocks();
        if (noRamBlocks() != noRamBlocksOther){
            return false;
        }
        else{
            for(int i = 0; i < noRamBlocks(); i++){
                try{
                    RAM hereRAM = getRamBlock(i);
                    RAM otherRAM = ((PC)o).getRamBlock(i);
                    if (!hereRAM.equals(otherRAM)){
                        return false;
                    }
                }
                catch(ModelException e){
                    throw new RuntimeException(e.getMessage());
                }
            }
        }
        return true;
    }

    /**
     * Creates a clone of the pc.
     * @see java.lang.Object#clone(java.lang.Object)
     */
    public Object clone(){
        PC copy;
        try {
            copy = new PC(getMake(), getModel());
            copy.addMainBoard((MainBoard)getMainBoard().clone());
            for (int i = 0;  i <  noCpus(); i++){
                copy.addCPU((CPU)getCPU(i).clone());
            }
            for (int j = 0;  j <  noHardDrives(); j++){
                copy.addHardDrive((HardDrive)getHardDrive(j).clone());
            }
            for (int k = 0;  k < noRamBlocks(); k++){
                copy.addRamBlock((RAM)getRamBlock(k).clone());
            }
        }
        catch (ModelException e){
            throw new RuntimeException(e.getMessage());
        }
        return copy;
    }

    /**
     * Overwrite the setPrice method.
     * @param price the price
     */
    public void setPrice(double price) throws ModelException{
         throw new ModelException("Price cannot be set on a PC it is calculated");
    }
    
    /**
     * Overwrite the getPrice method.
     * @return the calculated price of the PC
     */
    public double getPrice(){
        double price = 0;

        // Add price main board
        if (mainBoard != null){
          price += mainBoard.getPrice();
        }
        
        // Add price cpus
        Iterator cpuIter = cpus.iterator();
        while (cpuIter.hasNext()){
            CPU c = (CPU)cpuIter.next();
            price += c.getPrice();
        }

        // Add price hard drives
        Iterator diskIter = disks.iterator();
        while (diskIter.hasNext()){
            HardDrive d = (HardDrive)diskIter.next();
            price += d.getPrice();
        }

        // Add price RAM blocks
        Iterator ramIter = ramBlocks.iterator();
        while (ramIter.hasNext()){
            RAM r = (RAM)ramIter.next();
            price += r.getPrice();
        }
        return price;
    }

    /**
     * Adds the main board to the pc.
     * @param mainBoard the main board of the 
     */
    public void addMainBoard(MainBoard mainBoard){
        this.mainBoard = mainBoard;
    }

    /**
     * Adds a CPU to the pc.
     * @param cpu the cpu to add
     */
    public void addCPU(CPU cpu){
        cpus.add(cpu);
    }

    /**
     * Adds a disk to the pc.
     * @param disk the disk to add
     */
    public void addHardDrive(HardDrive disk){
        disks.add(disk);
    }
    
    /**
     * Adds a RAM block to the pc.
     * @param ram the ram block
     */
    public void addRamBlock(RAM ramBlock){
        ramBlocks.add(ramBlock);
    }
    
    /**
     * Gets the main board
     * @return the main board of the pc
     */
    public MainBoard getMainBoard(){
        return mainBoard;
    }
    
    /**
     *  Gets a cpu from the pc
     *  @no the cpu number (0..n-1)
     *  @return the no'th CPU in the pc
     */
    public CPU getCPU(int no) throws ModelException{
        if (no < 0) {
            throw new ModelException("CPU no must be positive");
        }
        if (no > cpus.size()){
            throw new ModelException("CPU no " + no + " too high. Only " + cpus.size() + " cpus");
        }
        return (CPU) cpus.get(no);
    }

    
    /**
     *  Gets a hard drive from the pc
     *  @no the hard drive number (0..n-1)
     *  @return the no'th hard drive in the PC
     */
    public HardDrive getHardDrive(int no) throws ModelException{
        if (no < 0) {
            throw new ModelException("Hard drive no must be positive");
        }
        if (no > disks.size()){
            throw new ModelException("Hard drive no " + no + " too high. Only " + disks.size() + " hard drives");
        }
        return (HardDrive) disks.get(no); 
    }

    /**
     *  Gets a ram block from the pc
     *  @no the hard drive number (0..n-1)
     *  @return the no'th hard drive in the PC
     */
    public RAM getRamBlock(int no) throws ModelException{
        if (no < 0) {
            throw new ModelException("RAM block no must be positive");
        }
        if (no > ramBlocks.size()){
            throw new ModelException("RAM block no " + no + " too high. Only " + ramBlocks.size() + " RAM blocks");
        }
        return (RAM)ramBlocks.get(no);
    }

    /**
     * Gets the number of cpus in the pc
     * @return number of cpus
     */
    public int noCpus(){
        return cpus.size();
    }
    
    /**
     * Gets the number of hard disks in the pc
     * @return number of hard disks
     */
    public int noHardDrives(){
        return disks.size();
    }

    /**
     * Gets the number of RAM blocks in the pc
     * @return number of ram blocks
     */
    public int noRamBlocks(){
        return ramBlocks.size();
    }

}
