package model;

import java.util.*;
import java.io.*;

/**
 * The warehouse class that controls all the various components that are sold by
 * the virual company MyHardware A/S. The class implements the Singleton design
 * pattern to avoid more than one ware house is created.
 * 
 * @author Kristian Torp, torp (at) cs (dot) aau (dot) dk
 * @version 1.0
 */
public class WareHouse implements Serializable {
	/** Collection of all components sold. */
	private Set components;

	/** The static field to the single instance of this class. */
	private static WareHouse warehouse = null;

	/**
	 * Constructor private to implement singleton design pattern.
	 */
	private WareHouse() {
		components = new HashSet();
	}

	/**
	 * Gets the single instance of the library.
	 * 
	 * @return the library
	 */
	public static WareHouse getWareHouse() {
		if (warehouse == null) {
			warehouse = new WareHouse();
		}
		return warehouse;
	}

	/**
	 * String representation of object.
	 * 
	 * @see java.lang.Object#toString(java.lang.Object)
	 */
	public String toString() {
		String res = "";
		Iterator iter = components.iterator();
		while (iter.hasNext()) {
			Object o = iter.next();
			res += o + "\n";
		}
		return res;
	}

	/**
	 * Checks if the ware house already contains the object
	 */
	public boolean contains(Object o) {
		// precondition
		if (o == null || !(o instanceof Component)) {
			return false;
		}

		Component comp = (Component) o;
		Iterator iter = components.iterator();
		while (iter.hasNext()) {
			Component other = (Component) iter.next();
			if (comp.idEquals(other)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Gets a component from the ware house with the specific make and model.
	 * 
	 * @param make
	 *            the make
	 * @param model
	 *            the model
	 */
	public Component get(String make, String model) {
		// precondition
		if (make == null || model == null) {
			return null;
		}

		Iterator iter = components.iterator();
		while (iter.hasNext()) {
			Component comp = (Component) iter.next();
			if (comp.getMake().equals(make) && comp.getModel().equals(model)) {
				return comp;
			}
		}
		return null;// found nothing
	}

	/**
	 * Addes a component to the components if it does not exists in advance.
	 * 
	 * @component the component to add.
	 */
	public void add(Component component) throws ModelException {
		// check that the cpu is not already in the
		Iterator iter = components.iterator();
		if (contains(component)) {
			throw new ModelException("Component already collection");
		}
		components.add(component);
	}

	/**
	 * Removes a component from the components sold
	 * 
	 * @component the component to remove
	 */
	public void remove(Component component) {
		// precondition
		if (component == null) {
			return;
		}

		Iterator iter = components.iterator();
		while (iter.hasNext()) {
			Component other = (Component) iter.next();
			if (component.idEquals(other)) {
				components.remove(other);
				break; // found what we are looking for so break
			}
		}
	}

	/**
	 * Removes a component via its make and model
	 */
	public void remove(String make, String model) throws ModelException {
		if (make == null) {
			throw new ModelException("Make not specified");
		}
		if (model == null) {
			throw new ModelException("Model not specified");
		}

		Component comp = get(make, model);
		if (comp != null) {
			remove(comp);
		}
	}

	/**
	 * Stores the data to disk.
	 * 
	 * @fileName the file name where to store
	 */
	public void store(String fileName) throws ModelException {
		try {
			FileOutputStream outStream = new FileOutputStream(fileName);
			ObjectOutputStream outObjectStream = new ObjectOutputStream(
					outStream);

			outObjectStream.writeObject(components);

			outObjectStream.flush();
			outObjectStream.close();
			outStream.close();
		} catch (IOException e) {
			throw new ModelException(e.getMessage());
		}
	}

	/**
	 * Loads a store to disk.
	 * 
	 * @fileName the file name where to load from
	 */
	public void load(String fileName) throws ModelException {
		FileInputStream inStream;
		ObjectInputStream inObjectStream;
		try {
			inStream = new FileInputStream(fileName);
			inObjectStream = new ObjectInputStream(inStream);

			components = (Set) inObjectStream.readObject();
			inObjectStream.close();
			inStream.close();

		} catch (FileNotFoundException e) {
			throw new ModelException(e.getMessage());
		} catch (IOException e) {
			throw new ModelException(e.getMessage());
		} catch (ClassNotFoundException e) {
			throw new ModelException(e.getMessage());
		}
	}
}