package ui;

import java.util.*;
import core.Console;
import model.*;

/**
 * The textual user interface for the ware house.
 * @author Kristian Torp, torp (at) cs (dot) aau (dot) dk.
 * @version 1.0
 */
public class UserInterface{
    /** String that clears the terminal screen under Unix/Linux */
    private static final String CLEAR=  "\033c";
    /** The library used */
    private WareHouse warehouse;
    
    /**
     * The default constructor.
     */
    public UserInterface(){
        warehouse = WareHouse.getWareHouse();
        System.out.println(CLEAR);
    }

    /**
     * Shows the main menu.
     * @return the selected menu
     */
    public String showMainMenu(){
        System.out.println("*** MyHardware A/S Main Menu ***");
        System.out.println();
        System.out.println("(A) Add component ");
        System.out.println("(D) Delete component");
        System.out.println("(S) Search component");
        System.out.println("(L) List all component");

        System.out.println();
        System.out.println("(R) Read from disk");
        System.out.println("(W) Write to disk");

        System.out.println();
        System.out.println("(X) eXit");
        System.out.println();
        return Console.readString();
    }

    /**
     * Handler for main menu
     */
    public void mainMenu(){
        String result;
        while(true){
            result = showMainMenu();
        
            if (result.equals("X")) break;
            else if (result.equals("A")) addComponent();
            else if (result.equals("D")) deleteComponent();
            else if (result.equals("S")) searchComponent();
            else if (result.equals("L")) list();
            else if (result.equals("R")) read();
            else if (result.equals("W")) write();
            else System.out.println("Unknown input");
       }
   }

   /**
    * Shows the add component menu
    * @return the select menu item
    */
    public String showAddReference(){
        System.out.println("*** Add Compoent ***");
        System.out.println();
        System.out.println("C) Add CPU ");
        System.out.println("H) Add hard drive ");
        System.out.println("M) Add main board");
        System.out.println("R) Add RAM block");
        System.out.println();
        System.out.println("P) Add PC");
        System.out.println();
        System.out.println("X) eXit");
        System.out.println();
        return Console.readString();
    }

    /**
     * Handles the add component menu
     */
    public void addComponent(){
        String result;
        while(true){
            result = showAddReference();
        
            if (result.equals("X")) break;
            else if (result.equals("C")) addCPU();
            else if (result.equals("H")) addHardDrive();
            else if (result.equals("M")) addMainBoard();
            else if (result.equals("R")) addRamBlock();
            else if (result.equals("P")) addPC();
            
            else System.out.println("Unknown input");
            waitAndClear();
       }
       System.out.println("Bye bye");
   }
  
  /**
   * Adds a CPU to the components.
   */
   public void addCPU(){
       try{
           warehouse.add(new CPU(getString("make"),
                                 getString("model"),
                                 getDouble("price"),
                                 getInt("frequence"),
                                 getString("unit")));
           System.out.println("CPU added!");
       } 
       catch (ModelException e){
           System.out.println("Cannot add CPU" + e);
       }
   }

  /**
    * Adds a hard drive to the components.
    */
   public void addHardDrive(){
       try{
           warehouse.add(new HardDrive(getString("make"),
                                       getString("model"),
                                       getDouble("price"),
                                       getInt("size"),
                                       getString("unit")));
           System.out.println("Hard drive added!");
       } 
       catch (ModelException e){
           System.out.println("Cannot add hard drive " + e);
       }
   }

  /**

  /**
   * Adds a main board to the components.
   */
   public void addMainBoard(){
       try{
           warehouse.add(new MainBoard(getString("make"),
                                       getString("model"),
                                       getDouble("price"),
                                       getString("information")));
           System.out.println("Main board added!");
       } 
       catch (ModelException e){
           System.out.println("Cannot add main board " + e);
       }
   }

  /**
   * Adds a ram block to the components.
   */
   public void addRamBlock(){
       try{
           warehouse.add(new RAM(getString("make"),
                                 getString("model"),
                                 getDouble("price"),
                                 getInt("size"),
                                 getString("unit")));
           System.out.println("RAM block added!");
       } 
       catch (ModelException e){
           System.out.println("Cannot add RAM block " + e);
       }
   }

   /**
    * Adds a PC to the components.
    */
   public void addPC(){
       try{
           PC pc = new PC(getString("make"),
                          getString("model"));
           warehouse.add(pc);
           addPCComponent(pc);                     
       } 
       catch (ModelException e){
           System.out.println("Cannot create PC " + e);
       }
   }

   /**
    * Shows the add component to PC menu
    * @return the select menu item
    */
    public String showAddPCComponent(){
        System.out.println("*** Add Component to PC ***");
        System.out.println();
        System.out.println("C) Add CPU ");
        System.out.println("H) Add hard drive ");
        System.out.println("M) Add main board");
        System.out.println("R) Add RAM block");
        System.out.println();
        System.out.println("X) eXit");
        System.out.println();
        return Console.readString();
    }

    /**
     * Handles the add component menu
     */
    public void addPCComponent(PC pc){
        String result;
        while(true){
            result = showAddPCComponent();
        
            if (result.equals("X")) break;
            else if (result.equals("C")) addCPUtoPC(pc);
            else if (result.equals("H")) addHardDrivetoPC(pc);
            else if (result.equals("M")) addMainBoardtoPC(pc);
            else if (result.equals("R")) addRamBlocktoPC(pc);
            else System.out.println("Unknown input");
            waitAndClear();
       }
   }
   
   /**
    * Adds a CPU to a PC.
    * @param pc the pc to add a CPU to
    */
   public void addCPUtoPC(PC pc){
        String make  = getString("Make  of CPU");
        String model = getString("Model of CPU");
        if (make != null && model != null){
            Component comp = warehouse.get(make, model);
            if (comp != null){
                if (comp instanceof CPU){
                    pc.addCPU((CPU)comp);
                    System.out.println("CPU added to pc");
                }
                else{
                    System.out.println("Not a CPU");
                }
            }
            else{
                System.out.println("No component found with make: " + make + " model: " + model);
            }
        }
        else{
            System.out.println("Make/model not specified");
        }
   }

   /**
    * Adds a HardDrive to a PC.
    * @param pc the pc to add a CPU to
    */
   public void addHardDrivetoPC(PC pc){
        String make  = getString("Make  of hard drive");
        String model = getString("Model of hard drive");
        if (make != null && model != null){
            Component comp = warehouse.get(make, model);
            if (comp != null){
                if (comp instanceof HardDrive){
                    pc.addHardDrive((HardDrive)comp);
                    System.out.println("Hard drive added to pc");
                }
                else{
                    System.out.println("Not a hard drive");
                }
            }
            else{
                System.out.println("No component found with make: " + make + " model: " + model);
            }
        }
        else{
            System.out.println("Make/model not specified");
        }
   }

   public void addMainBoardtoPC(PC pc){
        String make  = getString("Make  of main board");
        String model = getString("Model of main board");
        if (make != null && model != null){
            Component comp = warehouse.get(make, model);
            if (comp != null){
                if (comp instanceof MainBoard){
                    pc.addMainBoard((MainBoard)comp);
                    System.out.println("Main board added to pc");
                }
                else{
                    System.out.println("Not a main board");
                }
            }
            else{
                System.out.println("No component found with make: " + make + " model: " + model);
            }
        }
        else{
            System.out.println("Make/model not specified");
        }
   }

   public void addRamBlocktoPC(PC pc){
        String make  = getString("Make  of RAM block");
        String model = getString("Model of RAM board");
        if (make != null && model != null){
            Component comp = warehouse.get(make, model);
            if (comp != null){
                if (comp instanceof RAM){
                    pc.addRamBlock((RAM)comp);
                    System.out.println("RAM block added to pc");
                }
                else{
                    System.out.println("Not a RAM block");
                }
            }
            else{
                System.out.println("No component found with make: " + make + " model: " + model);
            }
        }
        else{
            System.out.println("Make/model not specified");
        }
   }


    /**
     * Deletes a component from the ware house.
     */
    public void deleteComponent(){
        String make  = getString("make to search for");
        String model = getString("model to search for");
        
        if (make != null && model != null){
            try{
                warehouse.remove(make, model);
                System.out.println("Make: " + make + " model: " + model + " deleted");
            }
            catch (ModelException e){
                System.err.println(e);
            }
        }
        else{
            System.out.println("No make/model specified");
        }
  }
   
   /**
     * Handles the search component menu.
     */
    public void searchComponent(){
        String make  = getString("make to search for");
        String model = getString("model to search for");
        if (make != null && model != null){
            Component comp = warehouse.get(make, model);
            if (comp != null){
                System.out.println(comp);
            }
            else{
                System.out.println("No component found with make: " + make + " model: " + model);
            }
        }
        else{
            System.out.println("Make/model not specified");
        }
  }

   /**
     * Lists all references.
     */
    public void list(){
        System.out.println(warehouse);
    }

   /**
     * Reads references from disk.
     */
    public void read(){
        String fileName = getString("File name to load from");
        if (fileName != null){
            try{
                warehouse.load(fileName);
                System.out.println("Components read from " + fileName);
            } catch (ModelException e){
                System.out.println(e);
            }
        }
        else{
            System.out.println("No file name specified");
        }
  }

   /**
     * Writes reference to disk.
     */
    public void write(){
        String fileName = getString("File name to write to");
        if (fileName != null){
            try{
                warehouse.store(fileName);
                System.out.println("Components written to " + fileName);
            } 
            catch (ModelException e){
                System.out.println(e);
            }
        }
        else{
            System.out.println("No file name specified");
        }
    }

    /**
     * Gets a string from the console.
     * @param text the text to ask for
     */
    public String getString(String text){
        System.out.println();
        return Console.readString("Enter " + text + ": ");
    }

    /**
     * Gets a double from the console.
     * @param text the text to ask for
     */
    public double getDouble(String text){
        System.out.println();
        return Console.readDouble("Enter " + text + " (double): ");
    }

    /**
     * Gets an integer from the console.
     * @param text the text to ask for
     */
    public int getInt(String text){
        System.out.println();
        return Console.readInt("Enter " + text + " (integer): ");
    }

    /**
     * Wait for user input and then clear the screen
     */
    public void waitAndClear(){
        System.out.println();
        Console.readString("Press any character to continue");
        System.out.println();
    }
}
